/***************************************************************************
                          album.cpp  -  description
                             -------------------
    begin                : Sat Dec 1 2001
    copyright            : (C) 2001 by Richard Groult
    email                : rgroult@jalix.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "album.h"

#include "directory.h"
#include "directoryview.h"
#include "imageviewer.h"
#include "imagelistview.h"
#include "mainwindow.h"
#include "albumimagefileiconitem.h"

#include <qtextstream.h>
#include <qfile.h>
#include <qapplication.h>

#include <klocale.h>

Album::Album(Directory * parent, const char *filename,
		DirectoryView* dirView, ImageViewer * iv,
		ImageListView * imageList, MainWindow * mw)
	:ListItem(parent, dirView, iv, imageList, mw),
	f(filename)
{
	full=QString(filename);
	size=QString("");
	init();
}

Album::~Album()
{
}

void
Album::init()
{
	setPixmap(0, BarIcon("imagegallery", KIcon::SizeSmall ));
	//list.setAutoDelete( TRUE );

}

void
Album::addURL(QStrList lst)
{
	QFile f(fullName());
	if (!f.open(IO_Raw | IO_ReadWrite | IO_Append) )
		return;
	QTextStream stream(&f);
	for (uint i = 0; i < lst.count (); i++)
	{
		stream << pathTo(QUriDrag::uriToLocalFile (lst.at (i))) << '\n';
	}
	
	if(size!="")size.sprintf ("%d", size.toInt()+lst.count());
	repaint();
	f.close();


}
	
QString 
Album::pathTo(QString fileName)
{
	int i=0;
	while(fileName[i] == fullName()[i] && (unsigned int)i<fileName.length() && (unsigned int)i< fullName().length())
		i++;
	if(fileName[i]=='/') i--;
	int pos=fileName.findRev('/', i, false);
	QString rFileName=fileName.right(fileName.length()-pos-1);
	QString rFullName=fullName().right(fullName().length()-pos-1);
	for(i=0; i<rFullName.contains('/', false); i++)
		rFileName="../"+rFileName;
	return rFileName;

}

QString
Album::text (int column) const
{
	if ( column == 0 )
		return QFileInfo(f).fileName();
	else
	if(column == 1 )
		return i18n("Album");
	else
		return size+"  ";
};

QString
Album::name ()
{
	return QFileInfo(f).baseName();
}

QString
Album::path ()
{
	return QString();
}

void
Album::load (bool )
{
	bool hasIm=imageList->hasImages();
	mw->setMessage(QString().sprintf(i18n("Loading album %s..."),(const char*)text(0)));
	QApplication::setOverrideCursor( waitCursor ); // this might take time
	if(mw->preview())
		imageList->stopLoading();

	QString currentPath=QFileInfo(fullName()).dirPath(true);
	QFile f(fullName());
	if (!f.open(IO_ReadOnly) )
		return;
	QTextStream ts(&f);
	QString lut;
	int i=0;
	AlbumImageFileIconItem *item;
	while(!ts.eof())
	{
		lut=currentPath+'/'+ts.readLine ();
		QFileInfo fi(lut);
		if(fi.exists())
		{
			item=new AlbumImageFileIconItem(imageList, this, QDir::cleanDirPath(lut), iv);
			list.append(item);
			i++;
		}
	}
	f.close();
	
	mw->slotAddImage(i);
	size.sprintf ("%d", i);
	iv->updateStatus();
	repaint();
	qApp->processEvents();

	if(mw->preview()) imageList->slotLoadFirst();
	imageList->sort();
	imageList->arrangeItemsInGrid () ;

	if(!hasIm)
	{
		if(dirView->loadFirstImage())
			imageList->first();
		else
			iv->loadImage(NULL);
	}
  
	QApplication::restoreOverrideCursor();	// restore original cursor
	mw->setMessage(i18n("Ready"));
}


void
Album::unLoad ()
{
	if(mw->preview())
		imageList->stopLoading();
	
	mw->slotRemoveImage(list.count());
	for (ImageFileIconItem *item = list.first(); item; item = list.next() )
		imageList->takeItem(item);
	list.clear();

	iv->updateStatus();
	imageList->slotUpdate();
	if(mw->preview())
		imageList->slotLoadFirst();
	//imageList->update();
}

bool 
Album::refresh(bool )
{
	qWarning("%s %d TODO Album::refresh(bool preview)",__FILE__,__LINE__);
	return true;
}

ListItem*
Album::find (const char *)
{
	return NULL;
}

void
Album::goTo (const char *)
{
}


void
Album::removeImage ( ListItem* )
{
}

void
Album::removeImage (AlbumImageFileIconItem *imf)
{
	list.remove(list.find(imf));
	imageList->takeItem(imf);
	mw->slotRemoveImage(1);

	size.sprintf ("%d", size.toInt()-1);
	repaint();
}

void
Album::create(QString )
{
}

void
Album::rename(QString )
{
}

void
Album::rename()
{
}

void
Album::properties()
{
}

bool
Album::add(QStrList )
{
	return false;
}

