/*
 * Copyright (C) 2000 Richard Groult <rgroult@jalix.org>
 * 		 2003 OGINO Tomonori <ogino@nn.iij4u.or.jp>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/*directory.cpp*/

#include "directory.h"

#include "directoryview.h"
#include "imageviewer.h"
#include "imagefileiconitem.h"
#include "dirfileiconitem.h"
#include "compressedimagefileiconitem.h"
#include "compressedfileitem.h"
#include "fileiconitem.h"
#include "imagelistview.h"
#include "mainwindow.h"
#include "directoryview.h"
#include "album.h"
#include "extract.h"

#include <unistd.h>

#include <qlistview.h>
#include <qstring.h>
#include <qdir.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qapplication.h>
#include <qpixmapcache.h> 

#include <kio/job.h>
#include <kiconloader.h>
#include <klineeditdlg.h>
#include <klocale.h>
#include <kstandarddirs.h>

Directory::Directory( DirectoryView* parent,
			 ImageViewer *iv,  ImageListView *imageList, MainWindow *mw )
	: ListItem(parent, iv, imageList, mw),
	f("/")
{
	p = NULL;
	full.append( "/" );
	init();
}

Directory::Directory(Directory* parent, const char * filename ,
			DirectoryView* dirView, ImageViewer *iv, ImageListView  *imageList, MainWindow *mw )
	: ListItem(parent, dirView, iv, imageList, mw),
	f(filename)
{
	p = parent;
	full.append(p->fullName()+f.name()+"/");
	init();
}


Directory::~Directory()
{
}


void
Directory::init()
{
	readable = TRUE;
	setPixmap(0, BarIcon(KMimeType::iconForURL(full), KIcon::SizeSmall ));
	size=-1;

}



ListItem*
Directory::find(const char* dir)
{
	QListViewItem* dirt;
	dirt=firstChild();
	while(dirt != 0)
	{
		if(QString(dir) == dirt->text(0)) break;
		dirt = dirt->nextSibling ();
	}
	if(dirt) return (ListItem*)dirt;
	return NULL;
}

void 
Directory::recursivelyOpen()
{
	setOpen(true);
	QListViewItem* dirt=firstChild();
	while(dirt != 0)
	{
		if(dirt->text(1)==i18n("Directory")) ((Directory*)dirt)->recursivelyOpen();
		dirt = dirt->nextSibling ();
	}
	qApp->processEvents();
}

void
Directory::setOpen( bool o )
{
	if ( o && !childCount() )
	{
		QApplication::setOverrideCursor( waitCursor ); // this might take time
		imageList->setUpdatesEnabled( FALSE );

		QString s( fullName() );
		QDir thisDir( s );
		if(dirView->showHiddenDir())
			thisDir.setFilter(QDir::All|QDir::Hidden);
		else
			thisDir.setFilter(QDir::All);
		if ( !thisDir.isReadable() )
		{
			readable = FALSE;
			QApplication::restoreOverrideCursor();  // restore original cursor
			imageList->setUpdatesEnabled( FALSE );
			//repaint();
			return;
		 }
		 const QFileInfoList * files = thisDir.entryInfoList();
		 if ( files )
		 {
		 	 QFileInfoListIterator it( *files );
		 	 QFileInfo * f;

		 	 while( (f=it.current()) != 0 )
		 	 {
					++it;
		 	  	  if ( f->fileName() == "." || f->fileName() == ".." )
		 	  	      ; // nothing
		 	  	  else
		 	  	  {
		 	  	  	  if ( f->isDir() || (f->isSymLink() && QFileInfo(f->readLink()).isDir()) )
		 	  	  	    	  (void)new Directory( this, f->fileName(), dirView, iv, imageList, mw);
		 	  	  	  else
		 	  	  	  {
		 	  	  	   	   QString ext =  QFileInfo(f->fileName()).extension().lower();
		 	  	  	   	   if ( Extract::canExtract(f->fileName()) )
		 	  	  	   	   	(void)new CompressedFileItem(this, f->fileName(), fullName(),  mw, dirView, iv, imageList);
		 	  	  	   	   else
		 	  	  	   	   if (ext=="sia")
		 	  	  	   	   	(void)new Album(this, f->absFilePath(), dirView, iv, imageList, mw);
		 	  	  	  }
		 	  	  }
		 	 }
			 dirView->sort();
		}
		if(!childCount()) setExpandable(false);
		QApplication::restoreOverrideCursor();      // restore original cursor

		dirView->startWatchDir(fullName());
		QListViewItem::setOpen( o );
	}
	else
	if(!o && childCount() )
	{
		QListViewItem *dirt=firstChild(), *oldDir;
		while(dirt != 0)
		{
			if(dirt->isSelected()) dirt->setSelected(false);
			dirt->setOpen(false);
			oldDir = dirt;
			dirt = dirt->nextSibling ();
			delete(oldDir);
		}
		QListViewItem::setOpen( false );
		dirView->stopWatchDir(fullName());
	}

	if ( o ){
		if(KMimeType::iconForURL(full )== "folder")
			setPixmap( 0, SmallIcon("folder_open") );
	}
	else{
		QString icon = KMimeType::iconForURL(full);
	 	setPixmap( 0, SmallIcon(icon) );
	}
	imageList->setUpdatesEnabled( TRUE );
}


bool
Directory::refresh(bool )
{
	QString s( fullName() );
	QDir thisDir( s );
	bool isS=isSelected();
	if(!isS)
	{
		thisDir.setFilter(QDir::Dirs);
	}
	else
	{
		thisDir.setFilter(dirView->filter());
	}
	const QFileInfoList * files = thisDir.entryInfoList();

	QStringList after;
	QStringList news;
	QStringList deleted;

	if ( files )
	{
		QFileInfoListIterator it( *files );
		QFileInfo * f;
		QString ext;

		QString filename;
		FileIconItem *item;
		while( (f=it.current()) != 0 )
		{
			++it;
			if(
				(f->isDir() && f->fileName()!=".")
				||
				(f->isFile () &&
				(dirView->showAllFile() ||
				(!dirView->showAllFile() && dirView->isImage(f)))))
			{
				item=imageList->findItem(f->fileName());
				if(!item) news.append(f->fileName());
				after.append(f->fileName());
			}
		}
	}
	else
		return false;

	FileIconItem *item;
	bool add=false, del=false;
	for ( item=list.first();  item!= 0; item=list.next() )
	{
		if(!after.contains(item->text()))
		{
			deleted.append(item->text());
			del=true;
		}
	}

	for ( QStringList::Iterator it = news.begin(); it != news.end(); ++it )
	{
		if(!QFileInfo(fullName()+*it).isDir ())
		{
			item= new ImageFileIconItem(imageList, this, *it, (const char*)fullName(), iv);
			list.append(item);
			mw->slotAddImage();
			size++;
			add=true;
		}
		else
		if(QFileInfo(*it).fileName()!="." && !find(*it))
		{
			if(QFileInfo(*it).fileName()!="..")
			{
				new Directory(this, *it, dirView, iv, imageList, mw);
				setExpandable( TRUE );
			}
			if(isS)
			{
				item= new DirFileIconItem(imageList, this, *it, (const char*)fullName(), iv);
				list.append(item);
				mw->slotAddImage();
				size++;
				add=true;
			}
		}
	}

	int toDel=0;
	for ( QStringList::Iterator it = deleted.begin(); it != deleted.end(); ++it )
	{
		dirView->removeDir(fullName()+*it);
		if(isS)
		{
			item=imageList->findItem(*it);
			list.remove(item);
			imageList->takeItem(item);
			toDel++;
		}
	}
	if(!del && !add) return false;
	if(isS)
	{
		if(!childCount()) setExpandable( false );
		size-=toDel;
		mw->slotRemoveImage(toDel);

		QPoint tl(imageList->contentsX(), imageList->contentsY());
		imageList->sort();
		imageList->setContentsPos(tl.x(), tl.y());
	}
	return true;
}

void
Directory::goTo(const char* )
{
}

void
Directory::setup()
{
	setExpandable( TRUE );
	QListViewItem::setup();
}


QString
Directory::name()
{
	return f.name();
}

QString
Directory::path()
{
	QDir dir=QFileInfo(fullName()).dir();
	if (dir.cdUp())
		return dir.absPath();
	else
		return QString();
}

QString
Directory::text( int column ) const
{
	if ( column == 0 )
		return f.name();
	else
	if(column == 2 )
	{
		if(size>=0)
			return QString::number(size)+"  ";
		else
			return "";
	}
	else
	if ( readable )
		return i18n("Directory");
	else
		return i18n("Locked");
}

void
Directory::loadFirst()
{
}


void
Directory::removeImage(ImageFileIconItem *imf)
{
	if(!imf)
	{
		return;
	}

	if(list.findRef(imf)!=-1)
	{
		list.removeRef(imf);
		imageList->takeItem(imf);
		mw->slotRemoveImage(1);
	}
	else
	{
	}
}

void
Directory::unLoad()
{
	if(mw->preview())
		imageList->stopLoading();

	int nbrI=0;
	for (FileIconItem *item = list.first(); item; item = list.next() )
	{
		//if(item->isImage())
			nbrI++;
		imageList->takeItem(item);
	}
	mw->slotRemoveImage(nbrI);
	list.clear();

	iv->updateStatus();
	imageList->slotUpdate();
	if(mw->preview()) imageList->slotLoadFirst();

	QPixmapCache::clear ();
}


void
Directory::load(bool )
{
	bool hasIm=imageList->hasImages();
	mw->setMessage(QString().sprintf(i18n("Loading directory %s..."),(const char*)text(0)));
	QApplication::setOverrideCursor( waitCursor ); // this might take time

	listView()->setSelected(this, TRUE);
	if(mw->preview())
		imageList->stopLoading();

	QString s( fullName() );
	QDir thisDir( s );
	thisDir.setFilter(dirView->filter());
	const QFileInfoList * files = thisDir.entryInfoList();
	if ( files )
	{
		QFileInfoListIterator it( *files );
		QFileInfo * f;
		QString ext;
		size=0;
		FileIconItem *item=NULL;

		while( (f=it.current()) != 0 )
		{
			++it;
			if(
				f->isFile () &&
					(dirView->showAllFile() ||
					(!dirView->showAllFile() && dirView->isImage(f))))
			{
				item = new ImageFileIconItem(imageList, this, f->fileName(), (const char*)fullName(), iv);
			}
			else
			if(f->isDir() && f->fileName()!=".")
			{
				item = new DirFileIconItem(imageList, this, f->fileName(), (const char*)fullName(), iv);
			}

			if(item)
			{
				list.append(item);
				//if(((DirectoryView*)listView())->isImage(f))
					size++;
			}
			item=NULL;
		}
		mw->slotAddImage(size);
		qApp->processEvents();
		imageList->sort();
		if(!hasIm)
		{
			if(dirView->loadFirstImage())
				imageList->first();
			else        
				iv->loadImage(NULL);
		}

		imageList->slotLoadFirst();
		iv->updateStatus();

		imageList->arrangeItemsInGrid ();
		imageList->slotUpdate() ;
	}
	
	QApplication::restoreOverrideCursor();	// restore original cursor
	mw->setMessage(i18n("Ready"));
}


void
Directory::createDir(QString dirName)
{
	KURL url = KURL("file://"+fullName()+dirName);
	KIO::mkdir( url );
	setExpandable(true);
	//if(isOpen())
	//	new Directory(this, dirName, dirView, iv, imageList, mw);
}

void
Directory::createAlbum(QString albumName)
{
	KURL urld = KURL("file://"+fullName()+albumName);
	KURL urlo = KURL("file://"+KStandardDirs().findResource("templates",".source/TextFile.txt"));
	KIO::copy(urlo, urld, false);
	setExpandable(true);
	if(isOpen())
		new Album(this, fullName()+albumName, dirView, iv, imageList, mw);
}

void
Directory::rename()
{
	if(newDirName!="")
	{	
		f.setName(newDirName);
		full=path()+"/"+newDirName+"/";
		mw->slotRefresh();
		repaint();
	}
}

void
Directory::rename(QString newDirName)
{
	this->newDirName=newDirName;

	QString oldN=fullName();
	QString newN=path()+"/"+newDirName;

	KURL orgName = KURL("file://"+oldN);
	KURL newName = KURL("file://"+newN);

	KIO::SimpleJob *renameJob=KIO::rename(orgName, newName, true );
	QObject::connect(renameJob, SIGNAL(result( KIO::Job *)),
		dirView, SLOT(renameDone( KIO::Job *)));
}

void
Directory::properties()
{
}








