/*
 * Copyright (C) 2000 Richard Groult <rgroult@jalix.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 /* directoryview.cpp */

#include "directoryview.h"

#include "describeAlbum.h"
#include "directory.h"
#include "replace.h"
#include "imageviewer.h"
#include "imagefileiconitem.h"
#include "compressedimagefileiconitem.h"
#include "imagelistview.h"
#include "mainwindow.h"
#include "album.h"

#include <qstring.h>
#include <qfile.h>
#include <qptrlist.h>
#include <qtimer.h>
#include <qprogressdialog.h>
#include <qdropsite.h>
#include <qdragobject.h>
#include <qtooltip.h>
#include <qdatastream.h>
#include <qmessagebox.h>
#include <qprogressdialog.h>
#include <qapplication.h>
#include <qdir.h>
#include <qclipboard.h>
#include <qpopupmenu.h>

#include <kurlrequesterdlg.h>
#include <kio/job.h>
#include <kio/jobclasses.h>
#include <kmessagebox.h>
#include <klineeditdlg.h>
#include <kglobalsettings.h>
#include <kpropertiesdialog.h>
#include <kfiledialog.h>
#include <klocale.h>
#include <kiconloader.h>
#include <klistview.h>
#include <kdirwatch.h>
#include <kapplication.h>
#include <kaction.h>
#include <kpopupmenu.h>

DirectoryView::DirectoryView (QWidget * parent, ImageViewer * iv, ImageListView* imageList, MainWindow * mw)
:KListView (parent, "DirectoryView")
{

	this->iv = iv;
	this->imageList = imageList;
	this->mw = mw;

	addColumn (i18n("Name"));
	addColumn (i18n("Type"));
	addColumn (i18n("Size"));

	setColumnAlignment( 1, Qt::AlignLeft );
	setColumnAlignment( 2, Qt::AlignRight );

	setAllColumnsShowFocus( TRUE );
	setShowSortIndicator( TRUE );
	setShowToolTips(TRUE);
	setFullWidth(TRUE);
	setSelectionMode (QListView::Extended);
	setAcceptDrops (TRUE);

	setSorting (1);
	sort();

	setShowHiddenDir(false);
	dropping = false;

	currentOperation=NONE;
	autoSkip=false;
	replaceAll=false;
	currentOperation=NONE;
	fileToProcess=NULL;
	destDir=NULL;
	clickedItem=NULL;
	
	image_pixmap=NULL;

	autoopenTime = 750;
	autoopen_timer = new QTimer (this);
	connect (autoopen_timer, SIGNAL (timeout ()), this, SLOT (openFolder ()));

	copyProgress=new QProgressDialog (mw, "copyProgress", true);

	dirWatch = new KDirWatch();
	connect(dirWatch, SIGNAL(dirty (const QString&)),  mw, SLOT(slotDirChange(const QString&)));
	startWatchDir();
}


void
DirectoryView::initMenu(KActionCollection *actionCollection)
{
	popup = new QPopupMenu();
	KPopupMenu *popupnew = new KPopupMenu ();
		actionCollection->action("editdirnew")->plug(popupnew);
		actionCollection->action("editalbumnew")->plug(popupnew);
	popup->insertItem(i18n("Create &New"), popupnew);
	actionCollection->action("dirRecOpen")->plug(popup);
	popup->insertSeparator();
	actionCollection->action("editdircopy")->plug(popup);
	actionCollection->action("editdirmove")->plug(popup);
	actionCollection->action("editdirrename")->plug(popup);
	actionCollection->action("editdirtrash")->plug(popup);
	actionCollection->action("editdirdelete")->plug(popup);
	popup->insertSeparator();
	actionCollection->action("editdirpaste files")->plug(popup);
	popup->insertSeparator();
	actionCollection->action("Dir Info")->plug(popup);
	actionCollection->action("Dir Properties")->plug(popup);
}


ListItem*
DirectoryView::getDir(QString dirfullname)
{
	QCString *res = new QCString (dirfullname);
	if((*res)[res->length()-1]=='/')
		res=new QCString(res->left(res->length()-1));

	int pos = res->find ("/");
	ListItem *ssrep;
	res->setStr (res->right (res->length () - pos - 1));
	pos = res->find ("/");
	ssrep = (Directory*)firstChild();

	while (pos != -1 && ssrep!=NULL)
	{
		ssrep = ssrep->find (res->left (pos));
		res->setStr (res->right (res->length () - pos - 1));
		if (ssrep)
			pos = res->find ("/");
		else
			break;
	}
	if(!res->isEmpty() && ssrep)
		ssrep = ssrep->find(*res);

	return ssrep;
}

void
DirectoryView::removeDir(QString dirfullname)
{
	delete(getDir(dirfullname));
}


void
DirectoryView::triggerUpdate ()
{
	KListView::triggerUpdate();
}


void
DirectoryView::updateContents ()
{
}

void 
DirectoryView::slotDirInfo()
{
	if(clickedItem)
	{
		   DescribeAlbum(mw, ((Directory*)clickedItem)->fullName(), "IDirInfo", true).exec();
	}
}

void
DirectoryView::slotDirProperty()
{
	if(clickedItem)
	{
		QApplication::setOverrideCursor (waitCursor);	
		KFileItem *item = new KFileItem(KURL(((Directory*)clickedItem)->fullName()), "inode/directory", S_IFDIR);
        	KPropertiesDialog *prop	= new KPropertiesDialog( item,
			mw->iv, "KPropertiesDialog",
			true, false);
			
		QApplication::restoreOverrideCursor ();
		prop->exec();
	}
}

bool
DirectoryView::isDropping()
{
	return dropping;
}

void
DirectoryView::slotShowItem (QListViewItem * item)
{
	ensureItemVisible(item);
}

void
DirectoryView::startWatchDir(QString dir)
{
	dirWatch->stopScan();
	dirWatch->addDir(dir);
	dirWatch->startScan();
}

void DirectoryView::stopWatchDir(QString dir)
{
	dirWatch->removeDir(dir);
}

void
DirectoryView::startWatchDir()
{
	  dirWatch->startScan();
}


void DirectoryView::stopWatchDir()
{
	dirWatch->stopScan();
}

void
DirectoryView::slotRightButtonPressed( QListViewItem *, const QPoint &, int )
{
}

void
DirectoryView::slotMouseButtonClicked( int , QListViewItem *, const QPoint &, int  )
{
}

void
DirectoryView::emitContextMenu (QListViewItem*, const QPoint&, int)
{
}

void
DirectoryView::contentsDragEnterEvent (QDragEnterEvent * event)
{
	dropping = true;
	setSelectionMode (QListView::Single);
	oldCurrent = selectedItem ();
	if (QTextDrag::canDecode (event))
	{
		event->acceptAction ();
		QListViewItem *i = itemAt (contentsToViewport (event->pos ()));
		if (i)
		{
			dropItem = i;
			autoopen_timer->start (autoopenTime);
		}
	}
}

void
DirectoryView::contentsDragLeaveEvent (QDragLeaveEvent *)
{
	autoopen_timer->stop ();
	dropItem = NULL;	
	setMultiSelection (false);
	setCurrentItem (oldCurrent);
	setSelected (oldCurrent, TRUE);
	setSelectionMode (QListView::Extended);
	dropping = false;
}

void
DirectoryView::contentsDragMoveEvent (QDragMoveEvent * e)
{
	if(!dropping)
		return;

	if (!QTextDrag::canDecode (e))
	{
		e->ignore ();
		return;
	}

	QListViewItem *i = itemAt (contentsToViewport ( e->pos ()));
	if (i)
	{
		setSelected (i, TRUE);
		e->acceptAction ();
		if (i != dropItem)
		{
			autoopen_timer->stop ();
			dropItem = i;			
			autoopen_timer->start (autoopenTime);
		}
	}
	else
	{
		e->ignore ();
		autoopen_timer->stop ();
		dropItem = NULL;
	}
}


void
DirectoryView::contentsDropEvent (QDropEvent * event)
{
	autoopen_timer->stop ();
	setSelected (dropItem, false);
	
	if ( !QUriDrag::canDecode(event) )
	{
		event->ignore();
	}
	else
	{
		event->acceptAction(true);
		imageList->stopLoading();

		QImage image;
		QStrList lst;

		if (QUriDrag::decode (event, lst))
		{
			qWarning("%s %d DirectoryView::contentsDropEvent (QDropEvent * event)",__FILE__,__LINE__);
			if (((ListItem*) dropItem)->text (1) == i18n("Album"))
			{
				((Album*)dropItem)->addURL(lst);
			}
			else
			if (((ListItem*) dropItem)->text (1) != i18n("Directory"))
			{
				dropItem = NULL;
				KMessageBox::error (this, i18n("Adding file in an archive is not yet implemented"), i18n("File(s) copy/move"));
			}
			else
			if(!QFileInfo(((Directory*)dropItem)->fullName()).isWritable ())
			{
				dropItem = NULL;
				KMessageBox::error (this, i18n("The destination directory is not writable"), i18n("File(s) copy/move"));
			}
			else
			switch (event->action ())
			{
				case QDropEvent::Copy:
				case QDropEvent::Move:
				{
					if(!dropItem)
					{
						qWarning("%s %d ERROR dropItem est null !!!", __FILE__,__LINE__);
		    				return;
					}
					destDir=new QString( ((Directory*)dropItem)->fullName());
	
					event->acceptAction();
					contentsDragLeaveEvent(NULL);
	
					fileToProcess = new QPtrList<QString>;
	
					QString *fdep = NULL;
					for (uint i = 0; i < lst.count (); i++)
					{
						fdep = new QString (QUriDrag::uriToLocalFile (lst.at (i)));
						fileToProcess->append(new QString (QUriDrag::uriToLocalFile (lst.at (i))));
					}
					copyProgress = new QProgressDialog (mw, "copyProgress", true);
					copyProgress->setTotalSteps(fileToProcess->count());
					copyProgress->setProgress(0);
					totalFiles=fileToProcess->count();
	
					if (event->action () == QDropEvent::Move)
					{
						currentItemName=imageList->currentItemName();

						currentOperation=MOVE;
						copyProgress->setLabelText(i18n("File(s) Move"));
						moveNext();

					}
					else
					{
						currentOperation=COPY;
						copyProgress->setLabelText(i18n("File(s) Copy"));
						copyNext();

					}
				}
		 		return;
		 		default:return;
			}
		}
	}

	setCurrentItem (oldCurrent);
	setSelected (oldCurrent, TRUE);

	dropping = false;
	setSelectionMode (QListView::Extended);
	
}    


QString*
DirectoryView::getDestName(QString *orgFile)
{
	QString *fdest = new QString (*destDir);         		    
	int pos = orgFile->findRev ("/");
	fdest->append(orgFile->right(orgFile->length() - pos));
	if (QDir(*orgFile) == QDir(*fdest))
	{
		QString rep;
		rep.sprintf(i18n("<br>Can't copy <b>%s</b> because the destination file is the same"),
				            (const char *) *orgFile);
		copyProgress->show();
		KMessageBox::error (copyProgress, rep, i18n("File(s) copy"));
		return NULL;
	}
	if (QFile(QFileInfo(*fdest).dirPath(true) + "/" + QFileInfo(*fdest).fileName() ).exists())
	{  		
		if(replaceAll)
		{
			return  fdest;
		}
		else
		{
		copyProgress->show();
		ConfirmReplace cr(copyProgress, "ConfirmReplace", *orgFile, *fdest);
		switch (cr.exec())
		{
			case 1 : /*Replace()*/
				return fdest;
            	   	        break;
				
			case 2 : /*slotSkip*/
				autoSkip=true;
				return NULL;
				break;
				
			case 3 :/*slotReplaceAll*/
				replaceAll=true;
				return fdest;
				break;
			
			case 4 : /*slotCancel*/
				setMultiSelection (false);
				clearSelection ();

				setCurrentItem (oldCurrent);
				setSelected (oldCurrent, TRUE);

				setSelectionMode (QListView::Extended);
				dropping = false;
				
				currentOperation=NONE;

			        return NULL;
				break;

			case 5 : /*slotRename*/
				return new QString(cr.getRename());
				break;
			}
		}
	}	
	return fdest;
}

void
DirectoryView::copyNext()
{
	if(!fileToProcess->isEmpty())
	{	
		copyProgress->setProgress(totalFiles-fileToProcess->count());
		QString *dest=getDestName(fileToProcess->first());

		if(dest)
		{
			copyProgress->setProgress(totalFiles-fileToProcess->count());
			copyProgress->setLabelText(i18n(QString().sprintf("Copy <b>`%s'</b>",(const char *)shrinkdn(*dest))));
		}

		while( (dest==NULL) && (!fileToProcess->isEmpty()) && (currentOperation!=NONE))
		{
			fileToProcess->removeFirst();
			if(!fileToProcess->isEmpty()) dest=getDestName(fileToProcess->first());
		}
		if(!fileToProcess->isEmpty())
		{
			if(currentOperation!=NONE)
			{
				QString *org=fileToProcess->first();
				copy(org, dest);
			}
		}
		else
		{
			currentOperation=NONE;
		}
	}
	else
	{
		currentOperation=NONE;
	}

	if(currentOperation==NONE)
	{
		currentOperation=NONE;
		delete(fileToProcess); fileToProcess=NULL;
		
		autoSkip=false;
  		replaceAll=false;

		copyProgress->setProgress(copyProgress->totalSteps());
		copyProgress->hide();
		delete(copyProgress);
		
		currentItemName=QString();
		
		delete(destDir); destDir=NULL;
		
		dropping = false;
		setSelectionMode (QListView::Extended);

		mw->slotRefresh();
		imageList->slotLoadFirst();
	}
}

void
DirectoryView:: moveNext()
{
	if(!fileToProcess->isEmpty())
	{
		QString *dest=getDestName(fileToProcess->first());
		if(dest)
		{
			copyProgress->setProgress(totalFiles-fileToProcess->count());
			copyProgress->setLabelText(i18n(QString().sprintf("Move <b>`%s'</b>",(const char *)shrinkdn(*dest))));
		}
		while( (dest==NULL) && (!fileToProcess->isEmpty()) && (currentOperation!=NONE))
		{
			fileToProcess->removeFirst();
			if(!fileToProcess->isEmpty()) dest=getDestName(fileToProcess->first());
		}
		if(!fileToProcess->isEmpty())
		{
			if(currentOperation!=NONE)
			{
				QString *org=fileToProcess->first();
				int pos = org->findRev ("/");
				const QString itfi(org->right (org->length () - (pos + 1)));
				FileIconItem *fi = imageList->findItem (itfi);
				if(fi)
				{
					FileIconItem *next=fi->nextItem();
					if(next)
						currentItemName=next->text();
					else
					{
						next=fi->prevItem();
						if(next)
							currentItemName=next->text();	
						else
							currentItemName=QString();
					}
				}
				else
					currentItemName=QString();
				if(fi && !fi->isMovable())
				{
					copy(org, dest);
				}
				else
				{
					move(org, dest);
				}
			}
		}
		else
		{
			currentOperation=NONE;
		}
	}
	else
	{
		currentOperation=NONE;
	}

	if(currentOperation==NONE)
	{
		currentOperation=NONE;
		delete(fileToProcess); fileToProcess=NULL;

		autoSkip=false;
  		replaceAll=false;
		
		copyProgress->setProgress(copyProgress->totalSteps());
		copyProgress->hide();
		delete(copyProgress);
		
		imageList->setCurrentItemName(currentItemName);
		
		currentItemName=QString();
				
		delete(destDir); destDir=NULL;
		
		//
		setCurrentItem (oldCurrent);
		setSelected (oldCurrent, TRUE);

		dropping = false;
		setSelectionMode (QListView::Extended);

		mw->slotRefresh();
		imageList->slotLoadFirst();
	}
}
void
DirectoryView::contentsMouseMoveEvent ( QMouseEvent * e )
{
	if(e->state () == LeftButton)
		;
	else
		QListView::contentsMouseMoveEvent(e);
}

void
DirectoryView::contentsMouseReleaseEvent(QMouseEvent* e)
{
	if (e->button () == RightButton)
	{
		clickedItem = itemAt(contentsToViewport(e->pos()));
		if (clickedItem->text (1) == i18n("Directory"))
		{
			popup->exec(e->globalPos());
		}
	}
}

void
DirectoryView::contentsMousePressEvent(QMouseEvent * e)
{
	if (e->button () == RightButton)
	{
		clickedItem = itemAt(contentsToViewport(e->pos()));
		if (clickedItem->text(1) == i18n("Directory"))
			popup->exec(e->globalPos());
	}
	else
	if (e->button () == LeftButton)
		QListView::contentsMousePressEvent (e);

	else
	if (e->button () == MidButton)
		contentsMouseDoubleClickEvent(e);

}

void
DirectoryView::contentsMouseDoubleClickEvent ( QMouseEvent * e )
{
	QListViewItem *item=itemAt(contentsToViewport(e->pos()));
	if(!item)
		return;
	if(item->isOpen())
		item->setOpen(false);
	else
		item->setOpen(true);
}

void
DirectoryView::copyingDirDone( KIO::Job *job)
{
	if(job->error()==0)
	{
		//pas d'erreurs
		ListItem* dest=getDir(dirDest);
		if(dest)
		{
			if(!dest->isOpen())
			{
				//rien a faire
			}
			else
			{
				QString name=QDir(dirOrg).dirName();
				if(!getDir(dirDest+name))
				{
					//ajout du rep
					if(dest->text(1)==i18n("Directory"))
					{
						(void)new Directory( (Directory*)dest, name, this, iv, imageList, mw);
					}
					dest->setExpandable(true);
				}
			}
		}
		else
		{
		}
	}
	else
	{
		//erreur
		qWarning("%s %d DirectoryView::copyingDirDone( KIO::Job *job)",__FILE__,__LINE__);
		job->showErrorDialog(copyProgress);
	}

}

void
DirectoryView::movingDirDone( KIO::Job *job)
{
	if(job->error()==0)
	{
		//pas d'erreurs
		ListItem* dest=getDir(dirDest);
		if(dest)
		{
			if(!dest->isOpen())
			{
				//rien a faire
			}
			else
			{
				QString name=QDir(dirOrg).dirName();
				if(!getDir(dirDest+name))
				{
					//ajout du rep
					if(dest->text(1)==i18n("Directory"))
					{
						(void)new Directory( (Directory*)dest, name, this, iv, imageList, mw);
					}
					dest->setExpandable(true);
				}
			}
		}
		else
		{
		}
		ListItem *d = getDir(dirOrg);
		if(d)
		{
			d->setOpen(false);
			d->setSelected(false);
			delete(d);
		}
	}
	else
	{
		//erreur
		qWarning("%s %d DirectoryView::movingDirDone( KIO::Job *job)",__FILE__,__LINE__);
		job->showErrorDialog(copyProgress);
	}
}

void
DirectoryView::copyingDone( KIO::Job *job)
{
	if(job->error()==0)
	{
		//pas d'erreurs
	}
	else
	{
		//erreur
		job->showErrorDialog();
	}
	fileToProcess->removeFirst();

	if(currentOperation==COPY)
		copyNext();
	else
		moveNext();
}

void
DirectoryView::movingDone( KIO::Job *job )
{
	if(job->error()==0)
	{

	}
	else
	{
		//erreur
		qWarning("%s %d DirectoryView::movingDone( KIO::Job *job)",__FILE__,__LINE__);
		job->showErrorDialog();
	}
	fileToProcess->removeFirst();

	if(currentOperation==MOVE)
		moveNext();
	else
		copyNext();
}

bool
DirectoryView::copy (QString *dep, QString *dest)
{
	KURL urldep,urldest;
	urldep.setPath(*dep);
	urldest.setPath(*dest);	
	KIO::FileCopyJob *fcj=KIO::file_copy(urldep, urldest, -1,
										true, false,
										false);
	connect(fcj, SIGNAL(result( KIO::Job *)),
	      this, SLOT(copyingDone( KIO::Job *)));

	return true;
}


bool
DirectoryView::move (QString *dep, QString *dest)
{
	KURL urldep,urldest;
	urldep.setPath(*dep);
	urldest.setPath(*dest);
	KIO::FileCopyJob *fcj=KIO::file_move(urldep, urldest, -1,
								true, false,
								false);
	connect(fcj, SIGNAL(result(KIO::Job *)), this, SLOT(movingDone( KIO::Job *)));
	return true;
}



void
DirectoryView::copy(QStrList uris, QString dest)
{
	if(!QFileInfo(QFileInfo(dest).dirPath()).isWritable())
	{
		 KMessageBox::error(mw->iv,QString().sprintf( i18n("<br>The directory '<b>%s</b>' is not writable"),(const char*)shrinkdn(dest)));
		 return;
	}

	this->destDir=new QString(dest);
	fileToProcess = new QPtrList<QString>;

	QString *fdep = NULL;
	for (uint i = 0; i < uris.count (); i++)
	{
		fdep = new QString (QUriDrag::uriToLocalFile(uris.at (i)));
		if(QFileInfo(*fdep).isFile())
		{
			fileToProcess->append(fdep);
		}
	}
	copyProgress = new QProgressDialog (mw, "copyProgress", true);
	copyProgress->setTotalSteps(fileToProcess->count());
	copyProgress->setProgress(0);
	totalFiles=fileToProcess->count();

	currentOperation=COPY;
	oldCurrent = selectedItem ();
	copyProgress->setLabelText(i18n("File(s) Copy"));
	copyNext();
}


void
DirectoryView::move(QStrList uris, QString dest)
{
	if(!QFileInfo(QFileInfo(dest).dirPath()).isWritable())
	{
		 KMessageBox::error(mw->iv,QString().sprintf( i18n("<br>The directory '<b>%s</b>' is not writable"),(const char*)shrinkdn(dest)));
		 return;
	}

	this->destDir=new QString(dest);
	fileToProcess = new QPtrList<QString>;
	
	QString *fdep = NULL;
	for (uint i = 0; i < uris.count (); i++)
	{
		fdep = new QString (QUriDrag::uriToLocalFile(uris.at (i)));
		if(QFileInfo(*fdep).isFile())
		{
			fileToProcess->append(fdep);
		}
	}
	copyProgress = new QProgressDialog (mw, "copyProgress", true);
	copyProgress->setTotalSteps(fileToProcess->count());
	copyProgress->setProgress(0);
	totalFiles=fileToProcess->count();

	currentOperation=MOVE;
	oldCurrent = selectedItem ();
	copyProgress->setLabelText(i18n("File(s) Move"));
	moveNext();
}



void
DirectoryView::openFolder ()
{
	autoopen_timer->stop ();
	if (dropItem && !dropItem->isOpen ())
	{
		setOpen(dropItem, true);
	}
}


void
DirectoryView::slotDirPasteFiles()
{
	QStrList uris;
	if(QUriDrag::decode(QApplication::clipboard()->data(), uris))
	{
		if(!uris.isEmpty())
			copy(uris, ((Directory*)clickedItem)->fullName());
	}

}



void
DirectoryView::slotRename ()
{
	slotRename(clickedItem);
}

void
DirectoryView::slotRename (QListViewItem *item)
{
	Directory *dir=(Directory*)item;
	QString fullname=dir->fullName();
	QString name=dir->name();	
	bool ok;
	const QString newName(KLineEditDlg::getText(QString().sprintf(i18n("Rename directory '<b>%s</b>' :"),(const char*)shrinkdn(fullname)),
				name,
				&ok, mw).stripWhiteSpace());
	if(ok && newName!="")
	{
		QString newDirName=dir->path()+"/"+newName;
		if(QFileInfo(newDirName).exists())
		{
			KMessageBox::error(mw->iv, QString().sprintf(i18n("<br>The directory '<b>%s</b>' already exists"), (const char*)shrinkdn(newDirName)));
			return;	
		}
		dir->rename(newName);		
	}
}
void
DirectoryView::renameDone( KIO::Job *job)
{
	if(job->error()==0)
	{
		//pas d'erreurs
		((Directory*)clickedItem)->rename();
	}
	else
	{
		//erreur
		job->showErrorDialog(mw);
	}

}

void
DirectoryView::slotNewAlbum()
{
	slotNewAlbum(clickedItem);
}


void
DirectoryView::slotNewAlbum (QListViewItem *item)
{
	bool ok;
	const QString newName(KLineEditDlg::getText((const QString&)QString().sprintf(i18n("Create new album in <b>%s</b>"),(const char*)shrinkdn(((Directory*)item)->fullName())),
				i18n("albumName"),
				&ok, mw).stripWhiteSpace());
	if(!ok || newName=="")
		return;

	KURL url("file:"+((Directory*)item)->fullName()+"/"+newName+".sia");
	if(url.fileName()!=NULL)
	{
		if(QFileInfo(url.path()).exists())
		{
		 	 KMessageBox::error(mw->iv, QString().sprintf(i18n("<br>The album <b>%s</b> already exists"), (const char*)url.fileName()));
		 	 return;
		}
		((Directory*)item)->createAlbum(url.fileName());
	}
}


void
DirectoryView::slotNewDir()
{
	slotNewDir(clickedItem);
}

void
DirectoryView::slotNewDir (QListViewItem *item)
{
	bool ok;
	const QString newName(KLineEditDlg::getText((const QString&)QString().sprintf(i18n("Create new directory in <b>%s</b>"),(const char*)shrinkdn(((Directory*)item)->fullName())),
				i18n("dirname"),
				&ok, mw).stripWhiteSpace());
	if(!ok || newName=="")
		return;

	KURL url("file:"+((Directory*)item)->fullName()+"/"+newName);
	if(url.fileName()!=NULL)
	{
		if(QFileInfo(url.path()).exists())
		{
		 	 KMessageBox::error(mw->iv, QString().sprintf(i18n("<br>The directory '<b>%s</b>' already exists"), (const char*)url.fileName()));
		 	 return;
		}
		((Directory*)item)->createDir(url.fileName());
	}
}

void
DirectoryView:: slotTrash()
{
	slotTrash(clickedItem);
}

void
DirectoryView::slotTrash (QListViewItem *item)
{    
	Directory *dir=(Directory*)item;
	if(checkTrash())
		if(KMessageBox::questionYesNo(mw->iv,
					QString().sprintf(i18n("<br>Do you really want to move to trash '<b>%s</b>'?"),(const char*)shrinkdn(dir->fullName())),
					i18n("Confirm deletion")) == KMessageBox::No)
			return;
	if(dir->isSelected()) dir->setSelected(false);
	dir->setOpen(false);
	KURL
		org("file:/"+dir->fullName()),
		dest("file:/"+KGlobalSettings::trashPath()+dir->text(0));

	KIO::file_move(org, dest, -1, true, false, false);
	delete(item);
}


void
DirectoryView:: slotSuppr()
{
	slotSuppr(clickedItem);
}

void
DirectoryView::slotSuppr (QListViewItem *item)
{
	Directory *dir=(Directory*)item;
	if(checkDelete())
		if(KMessageBox::questionYesNo(mw->iv, 
					QString().sprintf(i18n("<br>Do you really want to delete '<b>%s</b>'?"),(const char*)shrinkdn(dir->fullName())),
					i18n("Confirm deletion"))
			== KMessageBox::No)
			return;

	if(dir->isSelected()) dir->setSelected(false);
	dir->setOpen(false);
	KIO::del(KURL("file:/"+dir->fullName()));
	delete(item);
}

void
DirectoryView::recursivelyOpen()
{
	recursivelyOpen(clickedItem);
}

void
DirectoryView::recursivelyOpen(QListViewItem *item)
{
	((Directory*)item)->recursivelyOpen();
}


void
DirectoryView::slotDirCopy()
{
	Directory* item=(Directory*)clickedItem;
	if(!item)
		return;

		QString destDir=KFileDialog::getExistingDirectory(item->fullName(),
									mw,
									QString().sprintf(i18n("Copy directory %s to..."), (const char*)shrinkdn(item->fullName())));

	if(!destDir.isEmpty())
	{
		QString dest=destDir+"/";

		KURL
			urlorg("file:"+item->fullName()),
			urldest("file:"+dest);		
		dirOrg=item->fullName();
		dirDest=dest;
		
		KIO::CopyJob *job = KIO::copy(urlorg, urldest, true);
		connect(job, SIGNAL(result( KIO::Job *)), this, SLOT(copyingDirDone( KIO::Job *)));
	}   	
}

void 
DirectoryView::slotDirMove()
{
	Directory* item=(Directory*)clickedItem;
	if(!item)
		return;
	
	QString destDir=KFileDialog::getExistingDirectory(item->fullName(),
								mw,
								QString().sprintf(i18n("Move directory %s to..."), (const char*)shrinkdn(item->fullName())));
	
	if(!destDir.isEmpty())
	{
		QString dest=destDir+"/"+item->text(0);
		if(QFileInfo(dest).exists())
		{
		 	 KMessageBox::error(mw->iv, QString().sprintf(i18n("<br>The directory '<b>%s</b>' already exists"),(const char*)shrinkdn(dest)));
		 	 return;
		}

		if(!QFileInfo(QFileInfo(dest).dirPath()).isWritable())
		{
		 	 KMessageBox::error(mw->iv,QString().sprintf( i18n("<br>The directory '<b>%s</b>' is not writable"),(const char*)shrinkdn(dest)));
		 	 return;
		}
		
		KURL
			urlorg("file:"+item->fullName()),
			urldest("file:"+dest);
			
		dirOrg=item->fullName();
		dirDest=destDir+"/";
		
		KIO::FileCopyJob *job = KIO::file_move(urlorg, urldest, -1, true, false, true);
		connect(job, SIGNAL(result( KIO::Job *)), this, SLOT(movingDirDone( KIO::Job *)));
	}
}

bool
DirectoryView::checkTrash()
{
	return trash;
}

void
DirectoryView::setCheckTrash(bool trash)
{
	this->trash=trash;
}

bool
DirectoryView::checkDelete()
{
	return del;
}

void
DirectoryView::setCheckDelete(bool del)
{
	this->del=del;
}

bool
DirectoryView::checkShred()
{
	return shred;
}

void
DirectoryView::setCheckShred(bool shred)
{
	this->shred=shred;
}


void
DirectoryView::setShowHiddenDir(bool show)
{
	__showHiddenDir__=show;
}

bool
DirectoryView::showHiddenDir()
{
	return __showHiddenDir__;
}

void
DirectoryView::setLoadFirstImage(bool load)
{
	__loadfirstimage__=load;
}

bool
DirectoryView::loadFirstImage()
{
	return __loadfirstimage__;
}


void
DirectoryView::setShowHiddenFile(bool show)
{
	__showHiddenFile__=show;
}

bool
DirectoryView::showHiddenFile()
{
	return __showHiddenFile__;
}

void
DirectoryView::setShowAllFile(bool show)
{
	__showAllFile__=show;
}
void
DirectoryView::setShowDir(bool show)
{
	__showDir__=show;
}

bool
DirectoryView::showDir()
{
	return __showDir__;
}


int
DirectoryView::filter ()
{
	int fil;
	if(showHiddenFile())
		fil = QDir::Hidden|QDir::Files;
	else
		fil = QDir::Files;
	if(showDir())
		fil = fil | QDir::Dirs;
	return fil;
}

bool
DirectoryView::isImage(QFileInfo *info)
{
	QString ext = info->extension( FALSE ).lower();
	return			((ext == "jpg") ||  (ext == "jpeg") ||
					(ext == "gif") ||
					(ext == "png") ||
					(ext == "bmp") ||
					(ext == "tif") || (ext == "tiff")  ||
					(ext == "mng") ||
					(ext == "xpm") ||
					(ext == "xcf") ||
					(ext == "ico") ||
					(ext == "pbm") ||
					(ext == "eps") ||
					(ext == "krl") ||
					(ext == "pnm") ||
					(ext == "pgm") ||
					(ext == "ppm") ||
					(ext == "xbm") );
}


QString
DirectoryView::shrinkdn(QString str)
{
	const unsigned int len=20;
	if(str.length()<=len)
		return str;
	else
		return QString(str.left(len/2) + "..." + str.right(len/2));
}


QDragObject*
DirectoryView::dragObject ()
{
	return NULL;
}
