/***************************************************************************
                          dirfileiconitem.cpp  -  description
                             -------------------
    begin                : dim jun 23 2002
    copyright            : (C) 2002 by Richard Groult
    email                : rgroult@jalix.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/*
 * Copyright (C) 2000 Richard Groult <rgroult@jalix.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "dirfileiconitem.h"

#include "imageloader.h"
#include "directory.h"
#include "imageviewer.h"
#include "imagelistview.h"
#include "imagefileinfo.h"
#include "exif.h"

#include <qdragobject.h>
#include <qdropsite.h>
#include <qlistview.h>
#include <qstring.h>
#include <qfile.h>
#include <qpopupmenu.h>
#include <qfileinfo.h>
#include <qdir.h>
#include <qapplication.h>
#include <qdragobject.h>
#include <qmessagebox.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <qiconview.h>
#include <qpixmapcache.h>
#include <qurloperator.h>

#include <kiconloader.h>
#include <kfilemetainfo.h>
#include <kapplication.h>
#include <kpixmap.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kglobalsettings.h>
#include <konq_operations.h>
#include <kio/job.h>

DirFileIconItem::DirFileIconItem (ImageListView * parent,
				Directory * parentDir,
				const char *filename,
				const char *path,
				ImageViewer *iv,
				QString *description):
   FileIconItem (parent, path, filename, "folder", iv),
   f(filename)
{
	this->imageList=parent;
	this->parentDir = parentDir;
	this->description=description;
	
	QIconViewItem::setText(f.name());

	readable = TRUE;
	selected = false;

	full.append (path);
	full.append (filename);
	if(strcmp(filename,"..")==0)
		setSelectable(false);
	else
		__ismovable__=true;

	setRenameEnabled(false);

	info = new QFileInfo (full);

	size = new QString ();
	size->sprintf ("%c%020s", 0,(const char*)full);
	*size=size->lower();

	QDateTime epoch( QDate( 1980, 1, 1 ) );
	date = new QString ();
	date->sprintf("0%08d", epoch.secsTo(info->lastModified())) ;

	extension = new QString ("000"+full);

	type = new QString ("dir");

	pixLoad = false;
	doPreload = true;
	setName("DirFileIconItem");

//	setPixmap(BarIcon(fileInfo()->iconName(), 48));
	setPixmap(BarIcon(fileInfo()->iconName(), imageList->currentIconSize));	
	haspreview=true;

}


DirFileIconItem::~DirFileIconItem ()
{
}




void
DirFileIconItem::setText( const QString & text )
{
	if(text==this->text())
		return;

	QFileInfo *itemFileInfo = new QFileInfo( full );
	QDir dir( itemFileInfo->dir() );

	if(QFileInfo(itemFileInfo->dirPath(TRUE)+"/"+text).exists())
	{
		KMessageBox::error(iv, QString().sprintf(i18n("<br>The directory '<b>%s</b>' already exists"),(const char*)text));
		delete itemFileInfo;
		return;
	}

	if ( dir.rename( itemFileInfo->fileName(), text ) )
	{
		QString itemFileName = itemFileInfo->dirPath( TRUE ) + "/" + text;
		full = QCString(itemFileInfo->dirPath( TRUE ) + "/" + text);

		delete itemFileInfo;
		itemFileInfo = new QFileInfo( itemFileName );

		f.setName(text);
		QIconViewItem::setText( this->text() );
	}
	else
	{
		KMessageBox::error(iv, QString().sprintf(i18n("<br>The directory <b>%s</b> can't be renamed"), (const char*)text));
	}
	delete itemFileInfo;
}


bool
DirFileIconItem::suppression (bool suprFile)
{
	if (suprFile)
		return suppression ();
	else
	{
		//parentDir->removeImage (this);
		return true;
	}
}


bool
DirFileIconItem::suppression ()
{
	if (QDir().remove (fullName ()))
	{
		//parentDir->removeImage (this);
		return true;
	}
	else
	{
		return false;
	}
}


bool
DirFileIconItem::moveToTrash()
{
	KURL org("file:/"+fullName()), dest("file:/"+KGlobalSettings::trashPath()+getFullName(new QString(fullName())));
	KIO::file_move(org, dest, -1, true, false, false);
	return suppression(false);
};


bool
DirFileIconItem::shred()
{
	KIO::del(KURL("file:/"+fullName ()), true, false);
	suppression(false);
	return true;
}


QString
DirFileIconItem::fullName ()
{
	return full;
}



QString
DirFileIconItem::shrink(QString str, int len) const
{
	if(str.length()<=(unsigned int)len)
		return str;
	else
	{
		return str.left(len/2) + "..." + str.right(len/2);
	}
}

QString
DirFileIconItem::toolTipStr() const
{
	QString tip;
	ImageFileInfo iminfo((const char*)full, IMAGE, true);

	tip.sprintf("<b>name</b>: %s<br><b>location</b>: %s<br>%s<br>",
		(const char*)f.name(),
		(const char*)shrink(parentDir->fullName()),
		(const char*)(iminfo.hasInfo()?"<b>description</b>: "+iminfo.getTitle()+"<br>":QString()));

	
	tip+=fileInfo()->getToolTipText();
	return tip;
}



QString
DirFileIconItem::text (int column) const
{

	if (column == 0)
		return KIconViewItem::text();
	else if (column == 1)
		return (*extension);
	else if (column == 2)
		return (*size);
	else if (column == 3)
		return (*type);
	else
		return QString ();

}

void
DirFileIconItem::setKey ( const QString & k )
{
	if(k=="name")
		KIconViewItem::setKey("00000000000000000000"+text());
	else
	if(k=="size")
		KIconViewItem::setKey(*size);
	else
	if(k=="type")
		KIconViewItem::setKey(*extension);
	else
	if(k=="date")
		KIconViewItem::setKey(*date);
	else
		KIconViewItem::setKey(text());

}

