/*
 * Copyright (C) 2000 Richard Groult <rgroult@jalix.org>
 * 		 2003 OGINO Tomonori <ogino@nn.iij4u.or.jp>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* extract.cpp*/

#include "extract.h"

#include <kprocess.h>
#include <kio/job.h>

#include <kzip.h>
#include <kar.h>
#include <ktar.h>
#include <qdir.h>

#include <kurl.h>
#include <qvaluelist.h>
#include <kmessagebox.h>
#include <klocale.h>

void 
Extract::getEntryRecursive( const KArchiveDirectory * entry, const QString & path )
{
	QStringList const & entries(entry->entries());
	QStringList::const_iterator it = entries.begin();
	for(; it != entries.end(); ++it) {
		QString add = path + QChar('/') + *it;
		files += add;
		const KArchiveEntry * child = entry->entry(*it);
		if( child->isDirectory() ){ 
			const KArchiveDirectory * childdir;
			childdir = dynamic_cast<const KArchiveDirectory *>(child);
			getEntryRecursive( childdir, path + QChar('/') + *it );
		}
	}
}

Extract::Extract (const char *_filename)
{
	QFileInfo fileinfo ( _filename );

	QString  tmpdir;
	tmpdir.setNum (getpid ());
	tmpdir.prepend ("/tmp/.qlist");
	QDir().mkdir(tmpdir,TRUE);

	dest = tmpdir + '/' + fileinfo.fileName() + '/';
	ext = fileinfo.extension().lower();

	// Warn when the size of archive more than 'big(64MB).'
	QFile::Offset big = 0x4000000;// 64MB
	QFile qfile( _filename );
	if( qfile.size() > big ) {
		QString msg = QString( i18n("The size of selected archive seems to be TOO BIG!\nContinue?(size: %1MB)") ).arg( (qfile.size())>>20 );
		switch( KMessageBox::warningContinueCancel( 0, msg, i18n("Confirm")) ) {
			case KMessageBox::Cancel :
				return;
				break;
		}
	}

	// KZip - sync
	KArchive * arc=NULL;
	if ( ext == "zip" )
		arc = new KZip( fileinfo.absFilePath() );
	else if ( ext == "tar" || ext == "tar.gz" || ext == "tar.bz2" || ext == "tgz" )
		arc = new KTar( fileinfo.absFilePath() );
	else if ( ext == "ar" )
		arc = new KAr( fileinfo.absFilePath() );

	if ( arc->open( IO_ReadOnly ) ) {
		const KArchiveDirectory * dir = arc->directory();
	        dir->copyTo( dest );
		getEntryRecursive( dir, QString() );
	}
	files.sort();
};

void
Extract::slotEntryRcv (KIO::Job *, const KIO::UDSEntryList& entries)
{
	KIO::UDSEntryListConstIterator it;
	for(it = entries.begin(); it != entries.end(); ++it ){
		KIO::UDSEntry::const_iterator jt;
		for(jt = (*it).begin(); jt != (*it).end(); ++jt ){
			switch( (*jt).m_uds ) {
				case KIO::UDS_NAME:
					files.append( QString((*jt).m_str) );
					break;
				default:
					break;
			}
		}
	}
	emit finished(this);
}

bool
Extract::canExtract( const QString & _filename )
{
	QFileInfo info( _filename );
	QString ext = info.extension().lower();
	if( ext == "zip" || ext == "tar" || ext == "tar.gz" 
	 || ext == "tar.bz2" || ext == "tgz" || ext == "ar" )
		return true;
	return false;
}
