/*
 * Copyright (C) 2000 Richard Groult <rgroult@jalix.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* imageliste.cpp*/


#include "imagelistview.h"

#include "imageloader.h"
#include "mainwindow.h"
#include "kexifpropsplugin.h"
#include "qtfileicondrag.h"
#include "imagefileiconitem.h"
#include "dirfileiconitem.h"
#include "imageviewer.h"

#include <kprocess.h>
#include <kiconview.h>
#include <kfileiconview.h>
#include <kaccel.h>
#include <kaction.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kpropsdlg.h>
#include <kopenwith.h>
#include <klineeditdlg.h>
#include <kurlrequesterdlg.h>
#include <kglobalsettings.h>
#include <kio/job.h>
#include <kfiledialog.h>
#include <kfileitem.h>
#include <klocale.h>
#include <kpopupmenu.h>
#include <kiconeffect.h>
#include <kpixmapio.h>
#include <kcursor.h>
#include <kglobalsettings.h>
#include <konq_operations.h>


#include <qmessagebox.h>
#include <qevent.h>
#include <qmime.h>
#include <qiconview.h>
#include <qevent.h>
#include <qkeycode.h>
#include <qpopupmenu.h>
#include <qdragobject.h>
#include <qstringlist.h>
#include <qlistview.h>
#include <qapplication.h>

#include <stdlib.h>	

KToolTip::KToolTip(QWidget *parent, ImageListView *imageList)
: QToolTip(parent)
{
	this->imageList=imageList;
};


void
KToolTip::maybeTip(const QPoint &pos)
{
	FileIconItem *item = (FileIconItem*)imageList->itemAt(imageList->viewportToContents(pos));
	if(!item)
		return;
	QRect r = item->pixmapRect(false);
	r.moveTopLeft(imageList->contentsToViewport(r.topLeft()));
	if ( !r.isValid() )
		return;
	tip(r, item->toolTipStr());
}
void
KToolTip::tip ( const QRect & r, const QString & text )
{
	QToolTip::tip(r, text);
}












/////////////////////////

ImageListView::ImageListView (QWidget * parent, const char *name,
				MainWindow * mw)
	:KIconView (parent, name)
{
	this->mw = mw;

	popup = new KPopupMenu ();
	il = new ImageLoader ();

	sortMode=0;
	loop=true;
	p=true;
	random=false;
	curIt=NULL;
	dscr=NULL;

	connect(this, SIGNAL(selectionChanged()), SLOT(selectionChanged()));
	connect(this, SIGNAL(onItem(QIconViewItem*)), SLOT(highlight(QIconViewItem *)));
	connect(this, SIGNAL(onViewport()), SLOT(onViewport()));

	setResizeMode (Adjust);
	setWordWrapIconText(false);
	setShowToolTips(true);
	QIconView::setSelectionMode (Extended);
	setItemsMovable ( true );
	setItemTextPos( Bottom );
	setArrangement(LeftToRight);

	toolTips = new KToolTip(viewport(), this);
	iconEffect=new KIconEffect();
	
	this->currentIconSize=48;
}
void
ImageListView::contentsMouseMoveEvent ( QMouseEvent *e )
{
	KIconView::contentsMouseMoveEvent(e);
}

void
ImageListView::mouseMoveEvent(QMouseEvent * e )
{
	KIconView::mouseMoveEvent(e);
}

bool
ImageListView::doPreload()
{
	return p;
}

void
ImageListView::setPreload(bool p)
{
	this->p=p;
}

bool
ImageListView::doLoop()
{
	return loop;
}

void
ImageListView::setLoop(bool loop)
{
	this->loop=loop;
}

bool
ImageListView::doRandom()
{
	return random;
}

void
ImageListView::setRandom(bool ran)
{
	this->random=ran;
}

FileIconItem*
ImageListView::itemAt(QPoint pos)
{
	return (FileIconItem*)QIconView::findItem(pos);
}

void
ImageListView::setThumbnailSize(QSize newSize)
{


	if(itemTextPos()==Right)
		setGridX(newSize.width()+200);
	else
		setGridX(newSize.width()+5);
	setGridY(newSize.height()+15);
	il->setThumbnailSize(newSize);

	if(newSize.width()==80) currentIconSize=32;
	else if(newSize.width()==120) currentIconSize=48;
	else currentIconSize=64;


	for (FileIconItem *i = firstItem(); i; i=i->nextItem() )
	{
		i->setHasPreview(false);
		i->setPixmap(BarIcon(i->fileInfo()->iconName(), currentIconSize));
	}
 
}

void
ImageListView::slotUpdate ()
{
	//QIconView::slotUpdate();
	setContentsPos(0,0);
}

FileIconItem*
ImageListView::firstItem ()
{
	return (FileIconItem *) KIconView::firstItem ();
}

FileIconItem*
ImageListView::currentItem()
{
	return (FileIconItem *)KIconView::currentItem ();
}

FileIconItem*
ImageListView::lastItem()
{
	return (FileIconItem *)KIconView::lastItem ();
}

void
ImageListView::slotResetThumbnail()
{
	stopLoading();
	for (FileIconItem * item = firstItem (); item != 0; item = item->nextItem ())
		item->setPixmap(BarIcon (item->fileInfo()->iconName(), 48));
	arrangeItemsInGrid();
}


void
ImageListView::slotRename()
{
	if(KIconView::currentItem())
	{
		bool ok;
		QString name=((ImageFileIconItem*)KIconView::currentItem())->text(0);
		const QString newName(KLineEditDlg::getText(QString().sprintf(i18n("Rename '<b>%s</b>' :"),(const char*)name),
					 name,
					 &ok, mw->iv).stripWhiteSpace());
		if(ok && newName!="")
		{
			((ImageFileIconItem*)KIconView::currentItem())->setText(newName);
		}
	}
};

void
ImageListView::initMenu(KActionCollection *actionCollection)
{
	this->actionCollection=actionCollection;

	KPopupMenu *popupsort = new KPopupMenu ();
	actionCollection->action("by name")->plug(popupsort);
	actionCollection->action("by extension")->plug(popupsort);
	actionCollection->action("by size")->plug(popupsort);
	actionCollection->action("by date")->plug(popupsort);
	popup->insertItem(i18n("Sorting"), popupsort);

	KPopupMenu *popupar = new KPopupMenu ();
		KPopupMenu *popupIc = new KPopupMenu ();
		actionCollection->action("From left to right")->plug(popupIc);
		actionCollection->action("From top to bottom")->plug(popupIc);
		popupar->insertItem(i18n("Icons"), popupIc);

		KPopupMenu *popupTxt = new KPopupMenu ();
		actionCollection->action("on the right")->plug(popupTxt);
		actionCollection->action("at the bottom")->plug(popupTxt);
		popupar->insertItem(i18n("Text"), popupTxt);
	popup->insertItem(i18n("Arrangement"), popupar);

	actionCollection->action("FullScreen")->plug(popup);
	
	popup->insertSeparator();
	actionCollection->action("editcut")->plug(popup);
	actionCollection->action("editcopy")->plug(popup);
	actionCollection->action("editpaste")->plug(popup);

	popup->insertSeparator();
	actionCollection->action("rename")->plug(popup);
	actionCollection->action("edittrash")->plug(popup);
	actionCollection->action("editdelete")->plug(popup);
	actionCollection->action("editshred")->plug(popup);

	popup->insertSeparator();
	actionCollection->action("copyFilesTo")->plug(popup);
	actionCollection->action("moveFilesTo")->plug(popup);

	popup->insertSeparator();
	actionCollection->action("Open with Gimp")->plug(popup);
	actionCollection->action("Open with")->plug(popup);

	popup->insertSeparator();
	actionCollection->action("Properties")->plug(popup);
	actionCollection->action("Image Info")->plug(popup);
	actionCollection->action("Edit File Type")->plug(popup);
}


void
ImageListView::refresh ()
{
}

QString
ImageListView::currentItemName()
{
	if(currentItem())
		return currentItem()->text();
	else
		return QString();
}

void
ImageListView::setCurrentItemName(QString itemName, bool select)
{
	 KIconView::setCurrentItem(findItem(itemName));
	 if(currentItem())
	 {
	 	KIconView::setSelected(currentItem(), true, false);
	 	if(select) currentItem()->setSelected(true);
	}
}

bool
ImageListView::hasImages()
{
	return KIconView::count()!=0;
}

bool
ImageListView::hasImageSelected()
{
	if(!hasImages())
		return false;
	for (FileIconItem *i = firstItem(); i; i=i->nextItem() )
	{
		if(i->isSelected())
			return true;
	}
	return false;
}

void
ImageListView::selectionChanged()
{
	if(hasImageSelected())
		mw->setHasImageSelected(true);
	else
		mw->setHasImageSelected(false);
}

void
ImageListView::slotLoadFirst()
{
	stopLoading();
	if (mw->preview() && count()>0)
	{
		mw->slotReset ();
		imageLoading = firstItem();
		if(count()==1)
		{
			if(!imageLoading->hasPreview() && imageLoading->isImage())
				il->loadMiniImage (new QFileInfo(imageLoading->fullName()), this,  true);
			else imageLoading=NULL;

		}
		else
		{
			while(imageLoading && (imageLoading->hasPreview() || !imageLoading->isImage()))
			{
				imageLoading=imageLoading->nextItem ();
				mw->slotPreviewDone ();
			}
			if(imageLoading){ slotLoadNext ();}
		}
	}
	if(!imageLoading){ mw->slotDone();}
}

void
ImageListView::slotLoadFirst(FileIconItem *item)
{
	stopLoading();
	if (mw->preview())
	{
		mw->slotReset (false);
		imageLoading = item;
		if(imageLoading)
			slotLoadNext ();
		else
			mw->slotDone ();
	}
}

void
ImageListView::slotLoadNext ()
{
	il->loadMiniImage (new QFileInfo(imageLoading->fullName()), this,  true);
}


void
ImageListView::slotSetPixmap (const QPixmap pm)
{
	nbrTh++;

	if(imageLoading)
	{
		imageLoading->setPixmap (pm, true);
		mw->slotPreviewDone ();
		imageLoading = imageLoading->nextItem ();
		while(imageLoading && !imageLoading->isImage())
		{
			imageLoading = imageLoading->nextItem ();
		}
	}
	if (imageLoading)
		slotLoadNext ();
  else
	{
		mw->slotDone();
		//arrangeItemsInGrid();
	}
}


void
ImageListView::next ()
{
	if(!hasImages())
		return;

	FileIconItem *item=NULL;
	if(doRandom())
	{
		srand(time(NULL));
		while(item==NULL)
		{
			int x = (int)(rand()/(RAND_MAX+1.0)*contentsWidth());
			int y = (int)(rand()/(RAND_MAX+1.0)*contentsHeight());
			item=(FileIconItem *)KIconView::findItem((const QPoint&)QPoint(x,y));
		}
	}
	else
	{
		item=currentItem();
		if(!item)
			item=firstItem();
		else
			if(!currentItem()->isSelected())
			item=firstItem();
		else
			item=item->nextItem();
	}
	if(item)
	{
		while(item && !item->isImage())
			item=item->nextItem ();
		if(item)
		{
	 		KIconView::ensureItemVisible (item);
	 		KIconView::setCurrentItem (item);
	 		KIconView::setSelected (item, true, false);
	 		item ->setSelected (true);
			if(dscr)slotImageInfo();

		}
		else if(loop) first();
	}
	else if(loop) first();
}


void
ImageListView::previous ()
{
	if(!hasImages())
		return;
	
	FileIconItem *item=currentItem();
	if(!item)
		item=firstItem();
	else
	if(!currentItem()->isSelected())
		item=firstItem();
	else
		item=item->prevItem();

	if(item)
	{
		while(item && !item->isImage())
			item=item->prevItem ();
		if(item)
	 	{
			KIconView::ensureItemVisible (item);
	 		KIconView::setCurrentItem (item);
	 		KIconView::setSelected (item, true, false);
	 		item ->setSelected (true);
			if(dscr)slotImageInfo();

		}
		else if(loop) last();
	}
	else if(loop) last();
}

void
ImageListView::showEvent( QShowEvent * )
{
	KIconView::show();
}

void
ImageListView::contentsMouseReleaseEvent (QMouseEvent * e)
{
	if (e->button () == LeftButton)
	{
		KIconView::contentsMouseReleaseEvent ( e );
		int nbs=0;
		for (FileIconItem * item = firstItem (); item != 0; item = item->nextItem ())
		{
			if (item->isSelected () )
			{
				nbs++;
				if(nbs==2)
					return;
			}
		}
	}
	mousePress(e);
}


void
ImageListView::mousePress (QMouseEvent * e)
{
	if (e->button () == MidButton)
		contentsMouseDoubleClickEvent(e);
	else
	if (e->button () == LeftButton)
	{
		if(!KGlobalSettings::singleClick())
		{
			FileIconItem *item  = currentItem();
			if(item)
			{
				item->setSelected (true);
				if(dscr)slotImageInfo();
			}
		}
		else
		{
			if(!currentItem()) return;
			if(currentItem()->fileInfo()->mimetype().right(9)=="directory")
			{
				curIt=NULL;QApplication::restoreOverrideCursor ();
				mw->openDir(QDir::cleanDirPath(((FileIconItem*)currentItem())->fullName()));
			}
			else
			{
				FileIconItem *item  = currentItem();
				if(item)item ->setSelected (true);
			}
		}
	}
	else
	if (e->button () == RightButton)
		popup->exec (e->globalPos ());
}

void
ImageListView::contentsMouseDoubleClickEvent ( QMouseEvent * )
{
	if (!currentItem ()) return;
	if( currentItem()->isImage())
	{
		mw->slotFullScreen ();
		currentItem ()->setSelected (true);
	}
	else
	if(currentItem()->fileInfo()->mimetype().right(9)=="directory")
	{
		curIt=NULL;QApplication::restoreOverrideCursor ();
		mw->openDir(QDir::cleanDirPath(((FileIconItem*)currentItem())->fullName()));
	}
	else
		KRun::runURL("file://"+currentItem()->fullName(), currentItem()->fileInfo()->mimetype());

}

void
ImageListView::sort()
{
	switch(sortMode)
	{
		case 0:slotByName(); break;
		case 1:slotByExtension(); break;
		case 2:slotBySize(); break;
		case 3:slotByDate(); break;
	}
}


void
ImageListView::slotByName()
{
	sortMode=0;
	FileIconItem *i;
	for (i=(FileIconItem*)KIconView::firstItem(); i; i = (FileIconItem*)i->nextItem() )
	{
		i->setKey("name");
	}
	KIconView::sort();
}


void
ImageListView::slotByExtension()
{
	sortMode=1;
	FileIconItem *i;
	for (i=(FileIconItem*)KIconView::firstItem(); i; i = (FileIconItem*)i->nextItem() )
	{
		i->setKey("type");
	}
	KIconView::sort();
}


void
ImageListView::slotBySize()
{
	sortMode=2;
	FileIconItem *i;
	for (i=(FileIconItem*)KIconView::firstItem(); i; i = (FileIconItem*)i->nextItem() )
	{
		i->setKey("size");
	}
	KIconView::sort();
}


void
ImageListView::slotByDate()
{
	sortMode=3;
	FileIconItem *i;
	for (i=firstItem(); i; i = i->nextItem() )
	{
		i->setKey("date");
	}
	KIconView::sort();
}


QDragObject *
ImageListView::dragObject ()
{
	if ( !currentItem() )
	return 0;
	QPoint orig = viewportToContents( viewport()->mapFromGlobal( QCursor::pos() ) );
	QtFileIconDrag *drag = new QtFileIconDrag( viewport() );
	drag->setPixmap( *currentItem()->pixmap(),
				QPoint( currentItem()->pixmapRect().width() / 2, currentItem()->pixmapRect().height() / 2 ) );
	for ( FileIconItem *item = firstItem(); item; item = item->nextItem() )
	{
		if ( item->isSelected() )
		{
			QIconDragItem id;

			id.setData(QUriDrag::localFileToUri(item->fullName()));
			drag->append( id,
				QRect( item->pixmapRect( FALSE ).x() - orig.x(),
				item->pixmapRect( FALSE ).y() - orig.y(),
				item->pixmapRect().width(), item->pixmapRect().height() ),
				QRect( item->textRect( FALSE ).x() - orig.x(),
				item->textRect( FALSE ).y() - orig.y(),
				item->textRect().width(), item->textRect().height() ),
				QUriDrag::localFileToUri(item->fullName()));
		}
	}
    return drag;
}


void
ImageListView::slotWallpaper ()
{
	FileIconItem *item = currentItem ();
	if (!item)
		return;
	currentItem ()->setWallpaper ();
}


void
ImageListView::slotFileProperty()
{
	if(!currentItem())
		return;
	QApplication::setOverrideCursor (waitCursor);
	KFileItem *item = new KFileItem(*(currentItem()->fileInfo()));
	KPropertiesDialog *prop	= new KPropertiesDialog( item,
		mw->iv, "KPropertiesDialog",
		true, false);
	KEXIFPropsPlugin *exifProp = new KEXIFPropsPlugin(prop, currentItem()->fullName());
	prop->insertPlugin(exifProp);
	QApplication::restoreOverrideCursor ();
	prop->exec();
}


void
ImageListView::slotImageProperty()
{
}


void
ImageListView::slotImageInfo()
{
 	if(!currentItem()) return;

	QApplication::setOverrideCursor (waitCursor);
	if(!dscr)
	{
		dscr = new Describe(mw->iv, currentItem()->fullName(), "ImageInfo", false);
		connect(dscr, SIGNAL(close()), SLOT(slotDescribeClose()));
	}
	else
		dscr->setImageFile(currentItem()->fullName());
	QApplication::restoreOverrideCursor ();
	dscr->show();
}

void
ImageListView::slotDescribeClose()
{
	delete(dscr); dscr=NULL;
}

void
ImageListView::slotGimp ()
{
	FileIconItem *item = currentItem ();
	if (!item)
		return;

	KShellProcess *proc = new KShellProcess ();
	connect(proc, SIGNAL(processExited(KProcess *)), this, SLOT(slotEndGimp(KProcess *)));
	QString com;
	com.sprintf ("gimp-remote -n '%s' >/dev/null 2>/dev/null", (const char *) item->fullName());
	*proc << com;
	if(! proc->start ())
	{
		KRun::run("gimp",KURL::List(QStringList("file:/"+item->fullName())));
	}
}


void
ImageListView::slotEndGimp (KProcess *proc)
{
	if(proc->exitStatus()!=0)
	{
		FileIconItem* item = currentItem ();
		if (!item)
			return;
		KRun::run("gimp",KURL::List(QStringList("file:/"+item->fullName())));
	}
}



void
ImageListView::slotSupprimmer ()
{
	if(checkDelete())
		if(KMessageBox::questionYesNo(mw->iv, i18n("Do you really want to delete the selected file(s) ?"), i18n("Confirm deletion")) == KMessageBox::No)
			return;
	KURL::List list;
	QPtrList < FileIconItem > iconList;
	FileIconItem *nextItem = 0;
	for (FileIconItem * item = firstItem (); item != 0; item = item->nextItem ())
	{
		if (item->isSelected ())
		{
			nextItem = item->nextItem ();
			if (item->text (3) == "file")
			{
				list.append(KURL("file:/"+item->fullName()));
			}
			else
			{
				iconList.append(item);
			}
		}
	}
	if(!list.empty())
	{
		//KIO::DeleteJob *del=KIO::del( list, false, true );
		//connect(del, SIGNAL(result( KIO::Job *)), this, SLOT(deletionDone( KIO::Job *)));
		KonqOperations::del(this, KonqOperations::DEL, list);
	}
	for ( FileIconItem * item = iconList.first(); item; item = iconList.next() )
		item->suppression();


	if (nextItem)
	{
		KIconView::setCurrentItem (nextItem);
		nextItem->setSelected (true);
		KIconView::setSelected (currentItem(), true, true);
		ensureItemVisible (currentItem());
	}
}


void
ImageListView::deletionDone( KIO::Job *job)
{
	if(job->error()!=0) job->showErrorDialog();
	mw->slotRefresh();
}


void
ImageListView::slotMoveToTrash()
{
	if(checkTrash())
	if(KMessageBox::questionYesNo(mw->iv, i18n("Do you really want to move to trash the selected file(s) ?"), i18n("Confirm deletion")) == KMessageBox::No)
		return;
	FileIconItem *nextItem = 0;
	QPtrList < FileIconItem > iconList;
	KURL::List list;
	for (FileIconItem * item = firstItem (); item != 0; item = item->nextItem ())
	{
		if (item->isSelected ())
		{
			nextItem = item->nextItem ();
			if (item->text (3) == "file")
			{
				list.append(KURL("file:/"+item->fullName()));
			}
			else
			{
				iconList.append(item);
			}
		}
	}
	if(!list.empty())
	{
		//KIO::Job *m=KIO::move( list, KURL("file:/"+KGlobalSettings::trashPath()), true );
		//connect(m, SIGNAL(result( KIO::Job *)), this, SLOT(deletionDone( KIO::Job *)));
		KonqOperations::del(this, KonqOperations::TRASH, list);
	}
	for ( FileIconItem * item = iconList.first(); item; item = iconList.next() )
		item->moveToTrash();

	if (nextItem)
	{
		KIconView::setCurrentItem (nextItem);
		nextItem->setSelected (true);
		KIconView::setSelected (currentItem(), true, true);
		ensureItemVisible (currentItem());
	}
}


void
ImageListView::slotShred()
{
	if(checkShred())
	if(KMessageBox::questionYesNo(mw->iv, i18n("Do you really want to shred the selected file(s) ?"), i18n("Confirm deletion")) == KMessageBox::No)
		return;
	KURL::List list;
	QPtrList < FileIconItem > iconList;
	FileIconItem *nextItem = 0;
	for (FileIconItem * item = firstItem ();
	    item != 0; item =  item->nextItem ())
	{
		if (item->isSelected ())
		{
			nextItem = item->nextItem ();
			if (item->text (3) == "file")
			{
				list.append(KURL("file:/"+item->fullName()));
			}
			else
			{
				iconList.append(item);
			}
		}
	}
	if(!list.empty())
	{
		//KIO::DeleteJob *del=KIO::del( list, true, true );
		//connect(del, SIGNAL(result( KIO::Job *)), this, SLOT(deletionDone( KIO::Job *)));
		KonqOperations::del(this, KonqOperations::SHRED, list);
	}
	for ( FileIconItem * item = iconList.first(); item; item = iconList.next() )
		item->shred();

	if (nextItem)
	{
		KIconView::setCurrentItem (nextItem);
		nextItem->setSelected (true);
		KIconView::setSelected (currentItem(), true, true);

		ensureItemVisible (currentItem());
	}
}


void ImageListView::stopLoading ()
{
	il->stopLoading (true);
	nbrTh=0;
	mw->slotDone();
	arrangeItemsInGrid();
}


void ImageListView::first ()
{
	if(!hasImages())
	{
		mw->setEmptyImage();
		return;
	}
	FileIconItem *item=firstItem();
	while(item && !item->isImage())
		item=item->nextItem ();
	if(!item) {mw->setEmptyImage(); return;}
	KIconView::ensureItemVisible (item);
	KIconView::setCurrentItem (item);
	KIconView::setSelected (item, true, false);
	item ->setSelected (true);
	if(dscr)slotImageInfo();

}



void ImageListView::last ()
{
	if(!hasImages())
		return;

	FileIconItem *item=lastItem();
	while(item && !item->isImage())
		item=item->prevItem ();
	if(!item) return;
	KIconView::ensureItemVisible (item);
	KIconView::setCurrentItem (item);
	KIconView::setSelected (item, true, false);
	item ->setSelected (true);
	if(dscr)slotImageInfo();
}

FileIconItem*
ImageListView::findItem (const QString & text, bool fullname)
{
	for (FileIconItem *i=firstItem(); i; i=i->nextItem())
	{
		if(fullname)
		{
			if(i->fullName()==text)
				return i;
		}
		else
		if(i->text(0)==text)
			return i;
	}
	return NULL;
}

void ImageListView::slotOpenWith()
{
	FileIconItem *item = currentItem ();
	if (!item)
		return;
	if(mw->fullScreen())
		mw->slotFullScreen();
	KURL::List url((QStringList)item->fullName());
	KOpenWithDlg dialog(url, mw);
	if(dialog.exec()!=0)
		KRun::run(dialog.text(), KURL::List(QStringList("file:/"+item->fullName())));
};

void ImageListView::slotUnselectAll()
{
	clearSelection ();
}

void ImageListView::slotInvertSelection()
{
	KIconView::invertSelection () ;
}

void ImageListView::slotSelectAll()
{
	KIconView::selectAll(true);
}

bool
ImageListView::checkTrash()
{
	return trash;
}

void
ImageListView::setCheckTrash(bool trash)
{
	this->trash=trash;
}

bool
ImageListView::checkDelete()
{
	return del;
}

void
ImageListView::setCheckDelete(bool del)
{
	this->del=del;
}

bool
ImageListView::checkShred()
{
	return shred;
}

void
ImageListView::setCheckShred(bool shred)
{
	this->shred=shred;
}
void 
ImageListView::setPreloadIm(bool prel)
{
	__preload__=prel;
}
bool 
ImageListView::preloadIm()
{
	return __preload__;
}

void
ImageListView::setItemTextPos ( ItemTextPos pos )
{
	if(itemTextPos()==pos)
		return;
	if(pos==Bottom)
	{
		setGridX(gridX()-200+10);
		setWordWrapIconText (true);
}
	else
	{
		setGridX(gridX()+200-10);
		setWordWrapIconText (true);
	}
	KIconView::setItemTextPos(pos);
	arrangeItemsInGrid ();
}


void
ImageListView::slotFilesMoveTo()
{
	QStrList uris;
	for (FileIconItem* item = firstItem (); item != 0; item =  item->nextItem ())
	{
		if (item->isSelected () )
		{
			uris.append(QUriDrag::localFileToUri(item->fullName()));
		}
	}
	if(!uris.isEmpty())
	{
		QString destDir=KFileDialog::getExistingDirectory(mw->currentURL(), mw, i18n("Move selected files to ..."));
		if(!destDir.isEmpty())
		{
			mw->moveFilesTo(uris, destDir+"/");
		}
	}
}

void
ImageListView::slotFilesCopyTo()
{
	QStrList uris;
	for (FileIconItem* item = firstItem (); item != 0; item = item->nextItem())
	{
		if (item->isSelected () )
		{
			uris.append(QUriDrag::localFileToUri(item->fullName()));
		}
	}
	if(!uris.isEmpty())
	{
		QString destDir=KFileDialog::getExistingDirectory(mw->currentURL(), mw, i18n("Move selected files to ..."));
		if(!destDir.isEmpty())
		{
			mw->copyFilesTo(uris, destDir+"/");
		}
	}
}

void
ImageListView::highlight(QIconViewItem *item)
{
	if (curIt) onViewport();

	if(KGlobalSettings::changeCursorOverIcon()) QApplication::setOverrideCursor (KCursor::handCursor());
	if (! iconEffect->hasEffect(0,1)) return;
	currentIconItem = new QPixmap(*(item->pixmap()));
	curIt=item;
	item->setPixmap(iconEffect->apply(*item->pixmap(),0,1));
	repaintItem(item);
}


void
ImageListView::onViewport()
{
	if(curIt)
	{
		curIt->setPixmap(*currentIconItem);
		repaintItem(curIt);
		if(KGlobalSettings::changeCursorOverIcon()) QApplication::restoreOverrideCursor ();

		curIt=NULL;
	}
}

void
ImageListView::leaveEvent(QEvent * )
{
	onViewport();
}



